"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var echarts_1 = require("echarts");
var lodash_debounce_1 = __importDefault(require("lodash.debounce"));
var dom_ready_1 = __importDefault(require("../lib/dom-ready"));
var CircularAudioWave = /** @class */ (function () {
    function CircularAudioWave(audioContext, masterGain, colors) {
        var _this = this;
        if (colors === void 0) { colors = {}; }
        this.lastMaxR = 0;
        this.maxChartValue = 240;
        this.minChartValue = 100;
        this.playing = false;
        this.lineColorOffset = 0;
        var _a = this.createContainer(), container = _a.container, canvas = _a.canvas;
        this.container = container;
        this.canvas = canvas;
        dom_ready_1.default().then(function () {
            document.body.appendChild(container);
            if (document.body.clientWidth > 672) {
                _this.container.style.height = '200px';
                _this.container.style.textAlign = 'center';
                _this.container.style.boxShadow = '0 0 20px 1px rgba(255,255,255,0.7)';
                _this.canvas.style.width = '400px';
            }
        });
        this.chartOptions = this.buildChartOptions(colors);
        this.currentChartOptions = JSON.parse(JSON.stringify(this.chartOptions));
        this.context = audioContext;
        this.chart = echarts_1.init(this.canvas);
        this.chart.setOption(this.chartOptions, true);
        this._debouncedDraw = lodash_debounce_1.default(this._drawAnimation.bind(this), 25);
        if (!this.context) {
            console.warn('AUDIO LOGGER VISUALIZATION: No context could be found');
        }
        else {
            this.analyser = this.context.createAnalyser();
            masterGain.connect(this.analyser);
            // this.analyser.connect(audioContext.destination);
        }
    }
    CircularAudioWave.prototype.createContainer = function () {
        var container = document.createElement('div');
        var canvas = document.createElement('canvas');
        container.classList.add('circular-wave-container');
        container.style.position = 'fixed';
        container.style.bottom = '0px';
        container.style.left = '0px';
        container.style.width = '100vw';
        container.style.height = '25vh';
        container.style.opacity = '0';
        container.style.zIndex = '-1';
        container.style.transition = '0.5s opacity';
        container.style.backgroundColor = 'rgba(255,255,255,0.5)';
        canvas.classList.add('circular-wave-canvas');
        canvas.style.height = '100%';
        canvas.style.width = '100%';
        container.appendChild(canvas);
        return { canvas: canvas, container: container };
    };
    CircularAudioWave.prototype.start = function (bpm) {
        this.container.classList.add('started');
        this.container.style.opacity = '1';
        this.container.style.zIndex = '9999';
        this._setBpm(bpm);
        this._debouncedDraw();
    };
    CircularAudioWave.prototype.stop = function () {
        this.container.classList.add('stopped');
        this.container.style.opacity = '0';
        this.container.style.zIndex = '-1';
        if (this.currentAnimationFrame) {
            cancelAnimationFrame(this.currentAnimationFrame);
        }
        this.reset();
    };
    CircularAudioWave.prototype.destroy = function () {
        this.chart.dispose();
    };
    CircularAudioWave.prototype.reset = function () {
        this.currentChartOptions = JSON.parse(JSON.stringify(this.chartOptions));
    };
    CircularAudioWave.prototype._setBpm = function (bpm) {
        this.currentChartOptions.series[0].animation = false;
        if (this.currentChartOptions.series[2].rippleEffect) {
            this.currentChartOptions.series[2].rippleEffect.period = 150 / bpm;
        }
    };
    CircularAudioWave.prototype._drawAnimation = function () {
        if (!this.analyser) {
            return;
        }
        var freqData = new Uint8Array(this.analyser.frequencyBinCount);
        this.analyser.getByteFrequencyData(freqData);
        this._draw(freqData);
        requestAnimationFrame(this._debouncedDraw.bind(this));
    };
    CircularAudioWave.prototype._draw = function (freqData) {
        var _this = this;
        var waveData = this._generateWaveData(freqData);
        this.currentChartOptions.series[0].data = waveData.data;
        if (waveData.maxR > this.lastMaxR) {
            this.lastMaxR = waveData.maxR + 4;
        }
        else if (this.playing) {
            this.lastMaxR -= 2;
        }
        else {
            this.lastMaxR = this.minChartValue;
        }
        // maxbar
        this.currentChartOptions.series[1].data = Array.from(new Array(361), function (item, i) {
            return [_this.lastMaxR, i];
        });
        this.chart.setOption(this.currentChartOptions, true);
    };
    CircularAudioWave.prototype._generateWaveData = function (data) {
        var waveData = [];
        var maxR = 0;
        for (var i = 0; i <= 360; i++) {
            // (((OldValue - OldMin) * (NewMax - NewMin)) / (OldMax - OldMin)) + NewMin
            var freq = data[i];
            var r = ((freq - 0) * (this.maxChartValue - this.minChartValue)) / (255 - 0) +
                this.minChartValue;
            if (r > maxR) {
                maxR = r;
            }
            waveData.push([r, i]);
        }
        waveData.push([waveData[0][0], 360]);
        return {
            maxR: maxR,
            data: waveData
        };
    };
    CircularAudioWave.prototype.buildChartOptions = function (colors) {
        var _this = this;
        var _a = (colors || {}).wave, wave = _a === void 0 ? {} : _a;
        return {
            angleAxis: {
                type: 'value',
                clockwise: false,
                axisLine: {
                    show: false
                },
                axisTick: {
                    show: false
                },
                axisLabel: {
                    show: false
                },
                splitLine: {
                    show: false
                }
            },
            radiusAxis: {
                min: 0,
                max: this.maxChartValue + 50,
                axisLine: {
                    show: false
                },
                axisTick: {
                    show: false
                },
                axisLabel: {
                    show: false
                },
                splitLine: {
                    show: false
                }
            },
            polar: {
                radius: '100%'
            },
            series: [
                {
                    coordinateSystem: 'polar',
                    name: 'line',
                    type: 'line',
                    showSymbol: false,
                    lineStyle: {
                        color: {
                            colorStops: [
                                {
                                    offset: 0.7,
                                    color: colors.secondary || '#e91e63'
                                },
                                {
                                    offset: 0.3,
                                    color: colors.primary || '#3f51b5'
                                }
                            ]
                        },
                        shadowColor: wave.shadow || 'blue',
                        shadowBlur: 10
                    },
                    zlevel: 2,
                    data: Array.from(new Array(361), function (item, i) {
                        return [_this.minChartValue, i];
                    }),
                    silent: true,
                    hoverAnimation: false
                },
                {
                    coordinateSystem: 'polar',
                    name: 'maxbar',
                    type: 'line',
                    showSymbol: false,
                    lineStyle: {
                        color: wave.accent || '#87b9ca',
                        shadowColor: wave.accent || '#87b9ca',
                        shadowBlur: 10
                    },
                    data: Array.from(new Array(361), function (item, i) {
                        return [_this.minChartValue, i];
                    }),
                    silent: true,
                    hoverAnimation: false
                },
                {
                    coordinateSystem: 'polar',
                    name: 'interior',
                    type: 'effectScatter',
                    showSymbol: false,
                    data: [0],
                    symbolSize: 100,
                    rippleEffect: {
                        period: 3.5,
                        scale: 3
                    },
                    itemStyle: {
                        color: {
                            type: 'radial',
                            colorStops: [
                                {
                                    offset: 0,
                                    color: wave.accent || '#87b9ca'
                                },
                                {
                                    offset: 1,
                                    color: 'white'
                                }
                            ]
                        }
                    },
                    silent: true,
                    hoverAnimation: false,
                    animation: false
                }
            ]
        };
    };
    return CircularAudioWave;
}());
exports.CircularAudioWave = CircularAudioWave;
//# sourceMappingURL=circular-wave.js.map